/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package org.opengroup.osdu.notification.auth;

import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import org.opengroup.osdu.core.common.model.notification.GsaSecret;
import org.opengroup.osdu.core.common.model.notification.GsaSecretValue;
import org.opengroup.osdu.core.common.model.notification.Secret;
import org.opengroup.osdu.notification.auth.interfaces.SecretAuth;
import org.opengroup.osdu.notification.provider.interfaces.IGoogleServiceAccount;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import java.util.HashMap;
import java.util.Map;

@Component
public class GsaAuth implements SecretAuth {
    @Autowired
    private IGoogleServiceAccount gsaTokenProvider;

    private GsaSecret gsaSecret;

    public void setSecret(Secret secret) {
        this.gsaSecret = (GsaSecret) secret;
    }

    public Secret getSecret() {
        return this.gsaSecret;
    }

    public String getPushUrl(String endpoint) throws Exception {
        return endpoint;
    }

    public Map<String, String> getRequestHeaders() {
        Map<String, String> requestHeader = new HashMap<>();
        if (gsaSecret != null) {
            GsaSecretValue gsaSecretValue = gsaSecret.getValue();
            JsonParser jsonParser = new JsonParser();
            JsonElement root = jsonParser.parse(gsaSecretValue.getKey());
            String keyString = root.getAsJsonObject().toString();
            String idToken = this.gsaTokenProvider.getIdToken(keyString, gsaSecretValue.getAudience());
            requestHeader.put("Authorization", idToken);
        }
        return requestHeader;
    }
}
