/*
 *   Copyright 2017-2020, Schlumberger
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */

package org.opengroup.osdu.notification.di;

import com.google.api.client.util.Strings;
import org.opengroup.osdu.core.common.model.http.DpsHeaders;
import org.opengroup.osdu.core.common.model.http.RequestInfo;
import org.opengroup.osdu.notification.provider.interfaces.IPubsubRequestBodyExtractor;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Component;
import org.springframework.web.context.annotation.RequestScope;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.servlet.http.HttpServletRequest;
import java.util.Collections;
import java.util.Map;
import java.util.stream.Collectors;

@Component
@RequestScope
@Primary
public class RequestInfoExt extends RequestInfo {
    private HttpServletRequest httpRequest;

    @Inject
    private IPubsubRequestBodyExtractor requestBodyExtractor;

    @Inject
    public RequestInfoExt(HttpServletRequest request) {
        super(request);
        this.httpRequest = request;
    }

    @PostConstruct
    private void fixHeaders() {
        Map<String, String> headers = Collections
                .list(httpRequest.getHeaderNames())
                .stream()
                .collect(Collectors.toMap(h -> h, httpRequest::getHeader));
        this.setHeaders(DpsHeaders.createFromMap(headers));
    }

    public void assignPartitionIdIfNotInHeader() {
        if (!this.httpRequest.getMethod().equalsIgnoreCase("POST")) {
            return;
        }

        DpsHeaders headers = getHeaders();

        if (Strings.isNullOrEmpty(headers.getPartitionId())) {
            Map<String, String> attributes = this.requestBodyExtractor.extractAttributesFromRequestBody();
            String tenant = attributes.get(DpsHeaders.DATA_PARTITION_ID);
            headers.put(DpsHeaders.DATA_PARTITION_ID, tenant);
        }
    }
}
