// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.notification.provider.azure.messageBus.models;

import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class TopicSubscriptions {

    private Map<String, Map<String, List<String>>> existingTopicSubscriptions = new HashMap<>();
    private Map<String, Map<String, List<String>>> currentTopicSubscriptions = new HashMap<>();

    public boolean checkIfNewTopicSubscription(String partition, String sbTopicName, String subscriptionName) {

        if (existingTopicSubscriptions.get(partition) == null ||
                existingTopicSubscriptions.get(partition).get(sbTopicName) == null ||
                !existingTopicSubscriptions.get(partition).get(sbTopicName).contains(subscriptionName))
            return true;
        return false;
    }

    public void updateCurrentTopicSubscriptions(String partition, String sbTopicName, String subscriptionName) {
        // Update the active subscriptions
        currentTopicSubscriptions.putIfAbsent(partition, new HashMap<String, List<String>>());
        currentTopicSubscriptions.get(partition).putIfAbsent(sbTopicName, new ArrayList<>());
        currentTopicSubscriptions.get(partition).get(sbTopicName).add(subscriptionName);
    }

    public void clearTopicSubscriptions() {
        // Deletes the old subscriptions for next run
        this.existingTopicSubscriptions.clear();
        this.existingTopicSubscriptions.putAll(this.currentTopicSubscriptions);
        this.currentTopicSubscriptions.clear();
    }
}
